@@ -1,47 +1,41 @@
-# coding: utf8
 """
     weasyprint.images
     -----------------
 
     Fetch and decode images in various formats.
 
-    :copyright: Copyright 2011-2014 Simon Sapin and contributors, see AUTHORS.
-    :license: BSD, see LICENSE for details.
-
 """
 
-from __future__ import division, unicode_literals
-
-from io import BytesIO
 import math
+from io import BytesIO
+from xml.etree import ElementTree
 
 import cairocffi
-cairocffi.install_as_pycairo()  # for CairoSVG
-CAIRO_HAS_MIME_DATA = cairocffi.cairo_version() >= 11000
-
 import cairosvg.parser
 import cairosvg.surface
-assert cairosvg.surface.cairo is cairocffi, (
-    'CairoSVG is using pycairo instead of cairocffi. '
-    'Make sure it is not imported before WeasyPrint.')
+from PIL import Image
+
+from .layout.percentages import percentage
+from .logger import LOGGER
+from .urls import URLFetchingError, fetch
 
 try:
     from cairocffi import pixbuf
 except OSError:
     pixbuf = None
 
-from .urls import fetch, URLFetchingError
-from .logger import LOGGER
-from .compat import xrange
+assert cairosvg.surface.cairo is cairocffi, (
+    'CairoSVG is using pycairo instead of cairocffi. '
+    'Make sure it is not imported before WeasyPrint.')
 
 
 # Map values of the image-rendering property to cairo FILTER values:
 # Values are normalized to lower case.
-IMAGE_RENDERING_TO_FILTER = dict(
-    optimizespeed=cairocffi.FILTER_FAST,
-    auto=cairocffi.FILTER_GOOD,
-    optimizequality=cairocffi.FILTER_BEST,
-)
+IMAGE_RENDERING_TO_FILTER = {
+    'auto': cairocffi.FILTER_BILINEAR,
+    'crisp-edges': cairocffi.FILTER_BEST,
+    'pixelated': cairocffi.FILTER_NEAREST,
+}
 
 
 class ImageLoadingError(ValueError):
@@ -58,7 +52,7 @@ class ImageLoadingError(ValueError):
         return cls('%s: %s' % (name, value) if value else name)
 
 
-class RasterImage(object):
+class RasterImage:
     def __init__(self, image_surface):
         self.image_surface = image_surface
         self._intrinsic_width = image_surface.get_width()
@@ -67,22 +61,29 @@ class RasterImage(object):
             self._intrinsic_width / self._intrinsic_height
             if self._intrinsic_height != 0 else float('inf'))
 
-    def get_intrinsic_size(self, image_resolution):
+    def get_intrinsic_size(self, image_resolution, _font_size):
         # Raster images are affected by the 'image-resolution' property.
         return (self._intrinsic_width / image_resolution,
                 self._intrinsic_height / image_resolution)
 
     def draw(self, context, concrete_width, concrete_height, image_rendering):
-        if concrete_width > 0 and concrete_height > 0 and \
-                self._intrinsic_width > 0 and self._intrinsic_height > 0:
-            # Use the real intrinsic size here,
-            # not affected by 'image-resolution'.
-            context.scale(concrete_width / self._intrinsic_width,
-                          concrete_height / self._intrinsic_height)
-            context.set_source_surface(self.image_surface)
-            context.get_source().set_filter(
-                IMAGE_RENDERING_TO_FILTER[image_rendering])
-            context.paint()
+        has_size = (
+            concrete_width > 0
+            and concrete_height > 0
+            and self._intrinsic_width > 0
+            and self._intrinsic_height > 0
+        )
+        if not has_size:
+            return
+
+        # Use the real intrinsic size here,
+        # not affected by 'image-resolution'.
+        context.scale(concrete_width / self._intrinsic_width,
+                      concrete_height / self._intrinsic_height)
+        context.set_source_surface(self.image_surface)
+        context.get_source().set_filter(
+            IMAGE_RENDERING_TO_FILTER[image_rendering])
+        context.paint()
 
 
 class ScaledSVGSurface(cairosvg.surface.SVGSurface):
@@ -92,56 +93,92 @@ class ScaledSVGSurface(cairosvg.surface.SVGSurface):
     """
     @property
     def device_units_per_user_units(self):
-        scale = super(ScaledSVGSurface, self).device_units_per_user_units
+        scale = super().device_units_per_user_units
         return scale / 0.75
 
 
-class SVGImage(object):
-    def __init__(self, svg_data, base_url):
+class FakeSurface:
+    """Fake CairoSVG surface used to get SVG attributes."""
+    context_height = 0
+    context_width = 0
+    font_size = 12
+    dpi = 96
+
+
+class SVGImage:
+    def __init__(self, svg_data, base_url, url_fetcher):
         # Don’t pass data URIs to CairoSVG.
         # They are useless for relative URIs anyway.
         self._base_url = (
             base_url if not base_url.lower().startswith('data:') else None)
         self._svg_data = svg_data
+        self._url_fetcher = url_fetcher
 
-        # TODO: find a way of not doing twice the whole rendering.
         try:
-            svg = self._render()
+            self._tree = ElementTree.fromstring(self._svg_data)
         except Exception as e:
             raise ImageLoadingError.from_exception(e)
-        # TODO: support SVG images with none or only one of intrinsic
-        #       width, height and ratio.
-        if not (svg.width > 0 and svg.height > 0):
-            raise ImageLoadingError(
-                'SVG images without an intrinsic size are not supported.')
-        self._intrinsic_width = svg.width
-        self._intrinsic_height = svg.height
-        self.intrinsic_ratio = self._intrinsic_width / self._intrinsic_height
 
-    def get_intrinsic_size(self, _image_resolution):
-        # Vector images are affected by the 'image-resolution' property.
+    def _cairosvg_url_fetcher(self, src, mimetype):
+        data = self._url_fetcher(src)
+        if 'string' in data:
+            return data['string']
+        return data['file_obj'].read()
+
+    def get_intrinsic_size(self, _image_resolution, font_size):
+        # Vector images may be affected by the font size.
+        fake_surface = FakeSurface()
+        fake_surface.font_size = font_size
+        # Percentages don't provide an intrinsic size, we transform percentages
+        # into 0 using a (0, 0) context size:
+        # http://www.w3.org/TR/SVG/coords.html#IntrinsicSizing
+        self._width = cairosvg.surface.size(
+            fake_surface, self._tree.get('width'))
+        self._height = cairosvg.surface.size(
+            fake_surface, self._tree.get('height'))
+        _, _, viewbox = cairosvg.surface.node_format(fake_surface, self._tree)
+        self._intrinsic_width = self._width or None
+        self._intrinsic_height = self._height or None
+        self.intrinsic_ratio = None
+        if viewbox:
+            if self._width and self._height:
+                self.intrinsic_ratio = self._width / self._height
+            else:
+                if viewbox[2] and viewbox[3]:
+                    self.intrinsic_ratio = viewbox[2] / viewbox[3]
+                    if self._width:
+                        self._intrinsic_height = (
+                            self._width / self.intrinsic_ratio)
+                    elif self._height:
+                        self._intrinsic_width = (
+                            self._height * self.intrinsic_ratio)
+        elif self._width and self._height:
+            self.intrinsic_ratio = self._width / self._height
         return self._intrinsic_width, self._intrinsic_height
 
-    def _render(self):
-        # Draw to a cairo surface but do not write to a file.
-        # This is a CairoSVG surface, not a cairo surface.
-        return ScaledSVGSurface(
-            cairosvg.parser.Tree(
-                bytestring=self._svg_data, url=self._base_url),
-            output=None, dpi=96)
-
     def draw(self, context, concrete_width, concrete_height, _image_rendering):
-        # Do not re-use the rendered Surface object,
-        # but regenerate it as needed.
-        # If a surface for a SVG image is still alive by the time we call
-        # show_page(), cairo will rasterize the image instead writing vectors.
-        svg = self._render()
-        context.scale(concrete_width / svg.width, concrete_height / svg.height)
-        context.set_source_surface(svg.cairo)
-        context.paint()
+        if not concrete_width or not concrete_height:
+            return
+
+        try:
+            svg = ScaledSVGSurface(
+                cairosvg.parser.Tree(
+                    bytestring=self._svg_data, url=self._base_url,
+                    url_fetcher=self._cairosvg_url_fetcher),
+                output=None, dpi=96, output_width=concrete_width,
+                output_height=concrete_height)
+            if svg.width and svg.height:
+                context.scale(
+                    concrete_width / svg.width, concrete_height / svg.height)
+                context.set_source_surface(svg.cairo)
+                context.paint()
+        except Exception as e:
+            LOGGER.error(
+                'Failed to draw an SVG image at %s : %s', self._base_url, e)
 
 
-def get_image_from_uri(cache, url_fetcher, url, forced_mime_type=None):
+def get_image_from_uri(cache, url_fetcher, optimize_images, url,
+                       forced_mime_type=None):
     """Get a cairo Pattern from an image URI."""
     missing = object()
     image = cache.get(url, missing)
@@ -150,51 +187,67 @@ def get_image_from_uri(cache, url_fetcher, url, forced_mime_type=None):
 
     try:
         with fetch(url_fetcher, url) as result:
+            if 'string' in result:
+                string = result['string']
+            else:
+                string = result['file_obj'].read()
             mime_type = forced_mime_type or result['mime_type']
             if mime_type == 'image/svg+xml':
-                string = (result['string'] if 'string' in result
-                          else result['file_obj'].read())
-                image = SVGImage(string, url)
-            elif mime_type == 'image/png':
-                obj = result.get('file_obj') or BytesIO(result.get('string'))
-                try:
-                    surface = cairocffi.ImageSurface.create_from_png(obj)
-                except Exception as exc:
-                    raise ImageLoadingError.from_exception(exc)
-                image = RasterImage(surface)
+                # No fallback for XML-based mimetypes as defined by MIME
+                # Sniffing Standard, see https://mimesniff.spec.whatwg.org/
+                image = SVGImage(string, url, url_fetcher)
             else:
-                if pixbuf is None:
-                    raise ImageLoadingError(
-                        'Could not load GDK-Pixbuf. '
-                        'PNG and SVG are the only image formats available.')
-                string = (result['string'] if 'string' in result
-                          else result['file_obj'].read())
+                # Try to rely on given mimetype
                 try:
-                    surface, format_name = (
-                        pixbuf.decode_to_image_surface(string))
-                except pixbuf.ImageLoadingError as exc:
-                    raise ImageLoadingError(str(exc))
-                if format_name == 'jpeg' and CAIRO_HAS_MIME_DATA:
-                    surface.set_mime_data('image/jpeg', string)
-                image = RasterImage(surface)
+                    if mime_type == 'image/png':
+                        # Cairo already optimizes PNG images, no PIL needed
+                        try:
+                            surface = cairocffi.ImageSurface.create_from_png(
+                                BytesIO(string))
+                        except Exception as exception:
+                            raise ImageLoadingError.from_exception(exception)
+                        else:
+                            image = RasterImage(surface)
+                    else:
+                        image = None
+                except ImageLoadingError:
+                    image = None
+
+                # Relying on mimetype didn't work, give the image to GDK-Pixbuf
+                if not image:
+                    if pixbuf is None:
+                        raise ImageLoadingError(
+                            'Could not load GDK-Pixbuf. PNG and SVG are '
+                            'the only image formats available.')
+                    try:
+                        image = SVGImage(string, url, url_fetcher)
+                    except BaseException:
+                        if optimize_images:
+                            try:
+                                image = Image.open(BytesIO(string))
+                                output = BytesIO()
+                                image.save(
+                                    output, format=image.format, optimize=True)
+                            except BaseException:
+                                # Optimization did not work, keep the original
+                                pass
+                            else:
+                                string = output.getvalue()
+                        try:
+                            surface, format_name = (
+                                pixbuf.decode_to_image_surface(string))
+                        except pixbuf.ImageLoadingError as exception:
+                            raise ImageLoadingError(str(exception))
+                        if format_name == 'jpeg':
+                            surface.set_mime_data('image/jpeg', string)
+                        image = RasterImage(surface)
     except (URLFetchingError, ImageLoadingError) as exc:
-        LOGGER.warning('Failed to load image at %s : %s', url, exc)
+        LOGGER.error('Failed to load image at "%s" (%s)', url, exc)
         image = None
     cache[url] = image
     return image
 
 
-def percentage(value, refer_to):
-    """Return the evaluated percentage value, or the value unchanged."""
-    if value is None:
-        return value
-    elif value.unit == 'px':
-        return value.value
-    else:
-        assert value.unit == '%'
-        return refer_to * value.value / 100
-
-
 def process_color_stops(gradient_line_size, positions):
     """
     Gradient line size: distance between the starting point and ending point.
@@ -206,8 +259,8 @@ def process_color_stops(gradient_line_size, positions):
     Return processed color stops, as a list of floats in px.
 
     """
-    positions = [percentage(position, gradient_line_size)
-                 for position in positions]
+    positions = [
+        percentage(position, gradient_line_size) for position in positions]
     # First and last default to 100%
     if positions[0] is None:
         positions[0] = 0
@@ -229,7 +282,7 @@ def process_color_stops(gradient_line_size, positions):
         if position is not None:
             base = positions[previous_i]
             increment = (position - base) / (i - previous_i)
-            for j in xrange(previous_i + 1, i):
+            for j in range(previous_i + 1, i):
                 positions[j] = base + j * increment
             previous_i = i
     return positions
@@ -282,7 +335,7 @@ PATTERN_TYPES = dict(
     solid=cairocffi.SolidPattern)
 
 
-class Gradient(object):
+class Gradient:
     def __init__(self, color_stops, repeating):
         assert color_stops
         #: List of (r, g, b, a), list of Dimension
@@ -291,8 +344,8 @@ class Gradient(object):
         #: bool
         self.repeating = repeating
 
-    def get_intrinsic_size(self, _image_resolution):
-        # Raster images are affected by the 'image-resolution' property.
+    def get_intrinsic_size(self, _image_resolution, _font_size):
+        # Gradients are not affected by image resolution, parent or font size.
         return None, None
 
     intrinsic_ratio = None
@@ -463,7 +516,9 @@ class RadialGradient(Gradient):
     def _resolve_size(self, width, height, center_x, center_y):
         if self.size_type == 'explicit':
             size_x, size_y = self.size
-            return percentage(size_x, width), percentage(size_y, height)
+            size_x = percentage(size_x, width)
+            size_y = percentage(size_y, height)
+            return size_x, size_y
         left = abs(center_x)
         right = abs(width - center_x)
         top = abs(center_y)
